@@ -1,5 +1,6 @@
 #include <iostream>
 #include <string>
+//#define ARMA_DONT_PRINT_ERRORS
 #include <RcppArmadillo.h>
 #include "FBFsearch_H.h"
 
@@ -70,7 +71,7 @@ vec sub_elem_eq(vec v, vec w, double x)
   uvec iw;
 
   iw=find(w==x);
-  if(!iw.is_empty()){v=v.elem(iw);}else{v=math::nan();}
+  if(!iw.is_empty()){v=v.elem(iw);}else{v=datum::nan;}
    
   return v;
 
@@ -126,9 +127,8 @@ mat pow_vec(vec v, vec w)
 field<mat> add_to_tree(vec M, double lM, int nM, mat tree, double ltree)
 {
 
- int j,k,z,h,iM;
- 
- iM=0;
+ int j;
+ int k;
 
  field<mat> Res(2,1);
 
@@ -146,6 +146,10 @@ field<mat> add_to_tree(vec M, double lM, int nM, mat tree, double ltree)
 
  }
 
+ int z;
+ int h;
+ int iM=datum::nan;
+
  if(nM>0){ //if1
    
   z=0;
@@ -240,10 +244,10 @@ vec mov_tree(mat tree, vec M, int lM, vec vlM, int max_lM)
    mov2.elem(umov)=ones<vec>(mov.n_elem);
    mov=(mov2%M)%vlM;
    imov=find(mov>0);
-   if(!imov.is_empty()){mov=mov.elem(imov); mov=mov-1;} else{mov=math::nan();}
+   if(!imov.is_empty()){mov=mov.elem(imov); mov=mov-1;} else{mov=datum::nan;}
   }
 
- } else {mov=math::nan();}
+ } else {mov=datum::nan;}
  
 
  return mov;
@@ -275,9 +279,12 @@ double log_FBF_Ga_Gb(vec G_a, vec G_b, int edge, mat edges, mat YtY, int add, do
 
   int e1, e2, i, iwi;
   double p, b, S2, mu, sigma, logS2, ilogS2, logHhi, ilog4, log_num1, log_den1, log_w_1, log_num0i0, log_den0i0, log_w_0, log_FBF_unpasso;
-  vec V1, V2, G1, V11, pa1, pa0, betah, vv(1);
+  vec V1, V2, G1, V11, pa1, pa0, betah, vv(1), z1;
   uvec iw, ipa1;
-  mat e, yty, XtX, Xty, invXtX, _sigma;   
+  mat e, yty, XtX, Xty, invXtX;   
+  
+  // std::ostream nullstream(0);
+  // set_stream_err2(nullstream);
 
   e=edges.row(edge);
   e1=e(0);
@@ -286,14 +293,13 @@ double log_FBF_Ga_Gb(vec G_a, vec G_b, int edge, mat edges, mat YtY, int add, do
   V1=edges.col(0);
   V2=edges.col(1);
     
-   
   if(add==1){G1=G_a;}else{G1=G_b;}   
 
   V11=(V1+1)%G1;
   iw=find(V2==e2); pa1=V11.elem(iw);
   iw=find(pa1>0); pa1=pa1.elem(iw); pa1=pa1-1;
  
-  iw=find(pa1!=e1); if(!iw.is_empty()){pa0=pa1.elem(iw);}else{pa0=math::nan();}
+  iw=find(pa1!=e1); if(!iw.is_empty()){pa0=pa1.elem(iw);}else{pa0=datum::nan;}
  
   p=pa1.n_elem;
   b=(p+2*h+1)/n;
@@ -304,31 +310,41 @@ double log_FBF_Ga_Gb(vec G_a, vec G_b, int edge, mat edges, mat YtY, int add, do
    
   vv(0)=e2; Xty=sub_mat(YtY,pa1,vv);
   XtX=sub_mat(YtY,pa1,pa1);
-  invXtX=inv(sympd(XtX));
-  betah=invXtX*Xty;
+  betah=solve(XtX,Xty);
+  //invXtX=inv_sympd(XtX);
+  //betah=invXtX*Xty;
   S2=conv_to<double>::from(yty-(trans(Xty)*betah));
- 
-  iw=find(pa1==e1); mu=conv_to<double>::from(betah.elem(iw));
-  iwi=conv_to<int>::from(iw); _sigma=invXtX(iwi,iwi); sigma=conv_to<double>::from(_sigma);	
-   
-  log_w_1=(-n*(1-b)/2)*log(math::pi()*b*S2);
-  log_num1=-math::inf();
-  log_den1=-math::inf();
-  logS2=log(S2);
-
-  for(i=0; i<=h; i++){
-   
-   ilogS2=i*logS2;
-   logHhi=log_H_h_i(mu,sigma,h,i);
-   ilog4=-i*log(4);
- 
-   log_num1=log_add(log_num1, (ilog4+logHhi+lgamma((n-p-2*i)/2)+ilogS2));
-   log_den1=log_add(log_den1, (ilog4+logHhi+lgamma((n*b-p-2*i)/2)+ilogS2));
   
-  }
+  iw=find(pa1==e1); mu=conv_to<double>::from(betah.elem(iw));
+  iwi=conv_to<int>::from(iw); 
+  //_sigma=invXtX(iwi,iwi); sigma=conv_to<double>::from(_sigma);	
+  z1=zeros<vec>(pa1.n_elem);
+  z1(iwi)=1;
+  z1=solve(XtX,z1);
+  sigma=conv_to<double>::from(z1.elem(iw));
+  
+  if(S2>0){
+   log_w_1=(-n*(1-b)/2)*log(datum::pi*b*S2);
+   logS2=log(S2);
+   log_num1=-datum::inf;
+   log_den1=-datum::inf;
 
-  log_w_1=log_w_1+log_num1-log_den1;
+   for(i=0; i<=h; i++){
    
+    ilogS2=i*logS2;
+    logHhi=log_H_h_i(mu,sigma,h,i);
+    ilog4=-i*log(4);
+      
+    log_num1=log_add(log_num1, (ilog4+logHhi+lgamma((n-p-2*i)/2)+ilogS2));
+    log_den1=log_add(log_den1, (ilog4+logHhi+lgamma((n*b-p-2*i)/2)+ilogS2));
+  
+   }
+    
+   log_w_1=log_w_1+log_num1-log_den1;
+  }else{
+   log_w_1=datum::inf;
+  }
+     
   //calcolo w0
 
   if(!pa0.is_finite()){p=0;}else{p=pa0.n_elem;}
@@ -340,19 +356,25 @@ double log_FBF_Ga_Gb(vec G_a, vec G_b, int edge, mat edges, mat YtY, int add, do
   else{
    vv(0)=e2; Xty=sub_mat(YtY,pa0,vv);
    XtX=sub_mat(YtY,pa0,pa0);
-   invXtX=inv(sympd(XtX));
-   betah=invXtX*Xty;
+   betah=solve(XtX,Xty);
    S2=conv_to<double>::from(yty-(trans(Xty)*betah));
   }
 
-  log_w_0=(-(n*(1-b)/2))*log(math::pi()*b*S2)+log_num0i0-log_den0i0;
+  if(S2>0){
+   log_w_0=(-(n*(1-b)/2))*log(datum::pi*b*S2)+log_num0i0-log_den0i0;
+  }else{
+   log_w_0=datum::inf;
+  }
   
   //calcolo FBF
 
   if(add==1){log_FBF_unpasso=log_w_1-log_w_0;}
   else{log_FBF_unpasso=log_w_0-log_w_1;} 
-
-
+  
+  if(!is_finite(log_FBF_unpasso)){
+   log_FBF_unpasso=0;	  
+  }  
+ 
   return log_FBF_unpasso;
 
 } // end function
@@ -376,15 +398,15 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
    M_log_FBF=zeros<vec>(n_tot_mod); 
    log_num_MP=zeros<vec>(n_tot_mod); 
    M_q=zeros<vec>(lcv); 
-   tree=zeros<mat>(n_tot_mod*lcv,2); tree.fill(math::nan());
-   ltree=math::nan();
+   tree=zeros<mat>(n_tot_mod*lcv,2); tree.fill(datum::nan);
+   ltree=datum::nan;
    lM=lcv-1; 
  
-   sum_log_FBF=-math::inf(); 
-   log_sume=zeros<vec>(lcv); log_sume.fill(-math::inf());
+   sum_log_FBF=-datum::inf; 
+   log_sume=zeros<vec>(lcv); log_sume.fill(-datum::inf);
   
    M_log_RSMP=zeros<vec>(n_tot_mod);
-   sum_log_RSMP=-math::inf();
+   sum_log_RSMP=-datum::inf;
    imod_R=zeros<vec>(n_tot_mod);
 
    lM=lcv-1;
@@ -416,7 +438,7 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
     }
     
     M_q=exp(log_sume-sum_log_FBF);
-  
+   
     M_log_RSMP(t)=log_num_MP_G;
     sum_log_RSMP=log_add(sum_log_RSMP, log_num_MP_G);
    
@@ -433,7 +455,7 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
    
 
     pRSMP=exp(M_log_RSMP.subvec(0,limodR)-sum_log_RSMP);
-    pRSMP.max(i_n_mod_r);
+	pRSMP.max(i_n_mod_r);
     
     n_mod_r=imod_R(i_n_mod_r);
         
@@ -481,7 +503,7 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
         add=G(edge);   
        
         log_FBF1=log_FBF_Ga_Gb(G,G_t,edge,edges,YtY,add,nt,h);
-        log_FBF_G=log_FBF1+log_FBF_t;
+	    log_FBF_G=log_FBF1+log_FBF_t;
       
         M_G.col(t)=G;
         
@@ -500,7 +522,7 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
          if(G(imq)==1){log_sume(imq)=log_add(log_sume(imq), log_num_MP_G);}
         }
         M_q=exp(log_sume-sum_log_FBF);
-        
+	    
         limodR=limodR+1;
         imod_R(limodR)=t;
         M_log_RSMP(limodR)=log_num_MP_G; 
@@ -515,9 +537,13 @@ field<mat> FBF_heart(double nt, mat YtY, vec vG_base, double lcv, vec vlcv, mat
 
   t=t-1; 
   s=s-1;
-
+  
   M_P.subvec(0,t)=exp(log_num_MP.subvec(0,t)-sum_log_FBF);
-  M_P.subvec(0,t)=M_P.subvec(0,t)/sum(M_P.subvec(0,t));
+  if(max(M_P.subvec(0,t))>0){
+   M_P.subvec(0,t)=M_P.subvec(0,t)/sum(M_P.subvec(0,t));
+  }else{
+   M_P.subvec(0,t)=zeros<vec>(t+1);	  	  
+  }
   
   M_G=M_G.submat(0,0,lcv-1,t);
   
@@ -687,7 +713,7 @@ RcppExport SEXP FBF_RS(SEXP Corr, SEXP nobs, SEXP G_base, SEXP h, SEXP C, SEXP n
  M_G=heartRes(1,0);
  M_P=heartRes(2,0);
  
- iM_P=conv_to<vec>::from(sort_index(conv_to<vec>::from(M_P), 1));
+ iM_P=conv_to<vec>::from(sort_index(conv_to<vec>::from(M_P), "descend"));
 
  M_P2=zeros<vec>(n_hpp_c);
  M_G2=zeros<mat>(lcv,n_hpp_c);
@@ -786,7 +812,7 @@ RcppExport SEXP FBF_GS(SEXP Corr, SEXP nobs, SEXP G_base, SEXP h, SEXP C, SEXP n
     M_q2(edges(j,0),edges(j,1))=M_q(j);
   }
 
-  iM_P=conv_to<vec>::from(sort_index(conv_to<vec>::from(M_P), 1));
+  iM_P=conv_to<vec>::from(sort_index(conv_to<vec>::from(M_P), "descend"));
 
   M_P2=zeros<vec>(n_hpp_c);
   M_G2_j=zeros<mat>(lcv,n_hpp_c);
